@@ -10,9 +10,7 @@ setGeneric(
 #' Erosion prerequisites
 #'
 #' Calculates and writes capped slopes, L- and RUSLE S-factors (equations for
-#' summer conditions and slopes \eqn{\geq}{≥} 15 ft) to disk. Weighted flow
-#' accumulations less than one are set to one for the calculation of the
-#' L-factors.
+#' summer conditions and slopes \eqn{\geq}{≥} 15 ft) to disk.
 #'
 #' @param x An S4 [`RPhosFate-class`] river catchment object.
 #'
@@ -20,6 +18,10 @@ setGeneric(
 #'   the form of raster files.
 #'
 #' @references
+#' \cite{Desmet, P.J.J., Govers, G., 1996. A GIS procedure for automatically
+#' calculating the USLE LS factor on topographically complex landscape units.
+#' Journal of Soil and Water Conservation 51, 427–433.}
+#'
 #' \cite{Renard, K.G., Foster, G.R., Weesies, G.A., McCool, D.K., Yoder, D.C.,
 #' 1997. Predicting soil erosion by water: a guide to conservation planning with
 #' the Revised Universal Soil Loss Equation (RUSLE), Agriculture Handbook. U.S.
@@ -46,62 +48,65 @@ setMethod(
   "erosionPrerequisites",
   "RPhosFate",
   function(x) {
-    compareRaster(x@topo@rl_acc_wtd, x@topo@rl_slp, x@topo@rl_cha)
+    compareGeom(
+      x@topo@rl_acc_inf,
+      x@topo@rl_dir_inf,
+      x@topo@rl_slp_inf,
+      x@topo@rl_cha
+    )
 
     cs_dir_old <- setwd(file.path(x@cv_dir[1L], "Intermediate"))
     on.exit(setwd(cs_dir_old))
 
     # Capped slope in % (also relevant for transport)
-    x@topo@rl_slp_cap <- x@topo@rl_slp
+    x@topo@rl_slp_cap <- x@topo@rl_slp_inf
     x@topo@rl_slp_cap[x@topo@rl_slp_cap < x@parameters@ns_slp_min] <- x@parameters@ns_slp_min
     x@topo@rl_slp_cap[x@topo@rl_slp_cap > x@parameters@ns_slp_max] <- x@parameters@ns_slp_max
 
     # Capped slope in radian
-    rl_slp_cap_rad <- calc(
+    rl_slp_cap_rad <- app(
       x@topo@rl_slp_cap,
       function(x) {
         atan(x * 1e-2)
-      }
+      },
+      cores = x@is_ths
     )
 
-    # Weighted overland flow accumulation
-    rl_acc_wtd_ovl <- x@topo@rl_acc_wtd
-    rl_acc_wtd_ovl[!is.na(x@topo@rl_cha)] <- NA_real_
-    rl_acc_wtd_ovl[rl_acc_wtd_ovl < 1] <- 1
-
-    # Ratio of rill to interrill erosion
-    rl_LFa_b <- calc(
-      rl_slp_cap_rad,
-      function(x) {
-        sin(x) / (0.0896 * (3 * sin(x)^0.8 + 0.56))
-      }
-    )
-    # Rill erodibility parameter
-    rl_LFa_m <- calc(
-      rl_LFa_b,
-      function(x) {
-        x / (1 + x)
-      }
-    )
+    # Overland flow accumulation
+    rl_acc_inf_ovl <- x@topo@rl_acc_inf
+    rl_acc_inf_ovl[!is.na(x@topo@rl_cha)] <- NA_real_
 
     # L factor
-    is_res <- x@helpers@is_res
-    x@erosion@rl_LFa <- overlay(
-      x = rl_acc_wtd_ovl,
-      y = rl_LFa_m,
-      fun = function(x, y) {
-        ((x * is_res)^(1 + y) - ((x - 1) * is_res)^(1 + y)) / # nolint
-          (is_res * 22.13^y)
-      }
+    ns_siz <- x@helpers@ns_siz
+    ns_res <- x@helpers@ns_res
+
+    x@erosion@rl_LFa <- lapp(
+      c(x = rl_acc_inf_ovl, y = rl_slp_cap_rad, z = x@topo@rl_dir_inf),
+      fun = function(x, y, z) {
+        # Ratio of rill to interrill erosion
+        b <- sin(y) / (0.0896 * (3 * sin(y)^0.8 + 0.56))
+        # Rill erodibility parameter
+        m <- b / (1 + b)
+
+        z_rad <- z * pi / 180
+
+        ((x * ns_siz)^(m + 1) - ((x - 1) * ns_siz)^(m + 1)) /
+          (ns_res^(m + 2) * (abs(sin(z_rad)) + abs(cos(z_rad)))^m * 22.13^m)
+      },
+      cores = x@is_ths
     )
 
     # S factor
-    x@erosion@rl_SFa <- overlay(
-      x = rl_slp_cap_rad,
-      y = x@topo@rl_slp_cap,
+    x@erosion@rl_SFa <- lapp(
+      c(x = rl_slp_cap_rad, y = x@topo@rl_slp_cap),
       fun = function(x, y) {
-        ifelse(y < 9, 10.8 * sin(x) + 0.03, 16.8 * sin(x) - 0.5)
-      }
+        ifelse(
+          y < 9,
+          10.8 * sin(x) + 0.03,
+          16.8 * sin(x) - 0.5
+        )
+      },
+      cores = x@is_ths
     )
 
     x@topo@rl_slp_cap <- writeLayer(x, "slp_cap", x@topo@rl_slp_cap, "FLT8S")
@@ -159,8 +164,8 @@ setMethod(
   "erosion",
   "RPhosFate",
   function(x) {
-    compareRaster(
-      x@topo@rl_acc_wtd,
+    compareGeom(
+      x@topo@rl_acc_inf,
       x@erosion@rl_RFa,
       x@erosion@rl_KFa,
       x@erosion@rl_LFa,
@@ -178,7 +183,7 @@ setMethod(
       x@erosion@rl_LFa *
       x@erosion@rl_SFa *
       x@erosion@rl_CFa *
-      x@helpers@is_siz * 1e-4
+      x@helpers@ns_siz * 1e-4
 
     x@erosion@rl_ero <- writeLayer(x, "ero", x@erosion@rl_ero, "FLT8S")
 
@@ -226,8 +231,8 @@ setMethod(
   "RPhosFate",
   function(x, substance = "PP") {
     assertChoice(substance, slotNames(x@substances))
-    compareRaster(
-      x@topo@rl_acc_wtd,
+    compareGeom(
+      x@topo@rl_acc_inf,
       x@erosion@rl_ero,
       slot(x@substances, substance)@rl_xxc,
       x@topo@rl_clc
@@ -237,13 +242,16 @@ setMethod(
     on.exit(setwd(cs_dir_old))
 
     # Emission in kg/cell/yr
-    slot(x@substances, substance)@rl_xxe <- overlay(
-      x = x@erosion@rl_ero,
-      y = slot(x@substances, substance)@rl_xxc,
-      z = x@topo@rl_clc,
+    slot(x@substances, substance)@rl_xxe <- lapp(
+      c(
+        x = x@erosion@rl_ero,
+        y = slot(x@substances, substance)@rl_xxc,
+        z = x@topo@rl_clc
+      ),
       fun = function(x, y, z) {
         x * y * (1 + z * 1e-2) * 1e-3
-      }
+      },
+      cores = x@is_ths
     )
 
     slot(x@substances, substance)@rl_xxe <- writeLayer(
@@ -266,18 +274,13 @@ setGeneric(
 )
 #' Transport prerequisites
 #'
-#' Calculates hydraulic radii and determines cells representing inlets as well
-#' as riparian zones before writing them to disk.
+#' Determines cells representing inlets as well as riparian zones before writing
+#' them to disk.
 #'
 #' @inheritParams erosionPrerequisites,RPhosFate-method
 #'
 #' @inherit erosionPrerequisites,RPhosFate-method return
 #'
-#' @references
-#' \cite{Molnár, P., Ramírez, J.A., 1998. Energy dissipation theories and
-#' optimal channel characteristics of river networks. Water Resources Research
-#' 34, 1809–1818.}
-#'
 #' @seealso [`firstRun`], [`subsequentRun`]
 #'
 #' @examples
@@ -299,9 +302,9 @@ setMethod(
   "transportPrerequisites",
   "RPhosFate",
   function(x) {
-    compareRaster(
-      x@topo@rl_acc_wtd,
-      x@topo@rl_dir,
+    compareGeom(
+      x@topo@rl_acc_inf,
+      x@topo@rl_dir_inf,
       x@topo@rl_cha,
       x@topo@rl_rds
     )
@@ -309,151 +312,36 @@ setMethod(
     cs_dir_old <- setwd(file.path(x@cv_dir[1L], "Intermediate"))
     on.exit(setwd(cs_dir_old))
 
-    # Hydraulic radius in m
-    ns_rhy_a <- x@parameters@ns_rhy_a
-    ns_rhy_b <- x@parameters@ns_rhy_b
-    is_siz <- x@helpers@is_siz
-    x@transport@rl_rhy <- calc(
-      x@topo@rl_acc_wtd,
-      function(x) {
-        ns_rhy_a * (x * is_siz * 1e-6)^ns_rhy_b
-      }
+    # Riparian zone and inlet cells
+    li_rip_inl <- dinfRipInlCpp(
+      nm_dir_inf = as.matrix(x@topo@rl_dir_inf, wide = TRUE),
+      im_cha = as.matrix(x@topo@rl_cha, wide = TRUE),
+      im_rds = as.matrix(x@topo@rl_rds, wide = TRUE),
+      is_ths = x@is_ths
     )
 
-    # Riparian zone cells
-    x@topo@rl_rip <- raster(
-      dir_sth(
-        im_dir = as.matrix(x@topo@rl_dir),
-        im_sth = as.matrix(x@topo@rl_cha),
-        im_fDo = x@helpers@im_fDo
-      ),
-      template = x@topo@rl_acc_wtd
-    )
-
-    # Inlet cells
-    x@topo@rl_inl <- raster(
-      dir_sth(
-        im_dir = as.matrix(x@topo@rl_dir),
-        im_sth = as.matrix(x@topo@rl_rds),
-        im_fDo = x@helpers@im_fDo
-      ),
-      template = x@topo@rl_acc_wtd
-    )
-
-    # No inlet cells at channel cells
-    x@topo@rl_inl[!is.na(x@topo@rl_cha)] <- NA_integer_
-    # No riparian zone cells at road cells
-    x@topo@rl_rip[!is.na(x@topo@rl_rds)] <- NA_integer_
-    # No inlet cells at riparian zone cells
-    x@topo@rl_inl[!is.na(x@topo@rl_rip)] <- NA_integer_
+    x@topo@rl_rip <- rast(li_rip_inl$im_rip, crs = x@helpers@cs_cmt, extent = x@helpers@ex_cmt)
+    x@topo@rl_inl <- rast(li_rip_inl$im_inl, crs = x@helpers@cs_cmt, extent = x@helpers@ex_cmt)
+    set.names(x@topo@rl_inl, "inl")
 
     # Nearest channel cells for inlet cells
     df_out <- findNearestNeighbour(
-      rasterToPoints(x@topo@rl_inl),
-      rasterToPoints(x@topo@rl_cha),
-      x@helpers@ex_cmt
+      sort(as.points(x@topo@rl_inl), "inl"),
+      as.points(x@topo@rl_cha)
     )
 
     # X-coordinates of nearest channel cells to column numbers
-    df_out$Y.x <- (df_out$Y.x + x@helpers@is_res / 2 -
-      x@helpers@ex_cmt[1L]) / x@helpers@is_res
+    df_out$x <- colFromX(x@topo@rl_inl, df_out$x)
     # Y-coordinates of nearest channel cells to row numbers
-    df_out$Y.y <- x@helpers@is_rws - ((df_out$Y.y - x@helpers@is_res / 2 -
-      x@helpers@ex_cmt[3L]) / x@helpers@is_res)
+    df_out$y <- rowFromY(x@topo@rl_inl, df_out$y)
 
     # Substituting inlet values with integer codes identifying nearest channel
     # cells
-    df_out$code <- as.integer(df_out$Y.y * x@helpers@is_cls + df_out$Y.x)
-    # Bug in subs() {raster}: use default by and which
-    x@topo@rl_inl <- subs(x@topo@rl_inl, y = df_out[c(3L, 8L)])
+    df_out$code <- as.integer(df_out$y * x@helpers@is_cls + df_out$x)
+    x@topo@rl_inl <- subst(x@topo@rl_inl, df_out[["from_id"]], df_out[["code"]])
 
-    x@transport@rl_rhy <- writeLayer(x, "rhy", x@transport@rl_rhy, "FLT8S")
-    x@topo@rl_rip      <- writeLayer(x, "rip", x@topo@rl_rip     , "INT4S")
-    x@topo@rl_inl      <- writeLayer(x, "inl", x@topo@rl_inl     , "INT4S")
-
-    x
-  }
-)
-
-#### transportCalcOrder ####
-#' @export
-setGeneric(
-  "transportCalcOrder",
-  function(x, ...) standardGeneric("transportCalcOrder")
-)
-#' Transport calculation order
-#'
-#' Determines the cell transport calculation order.
-#'
-#' @inheritParams erosionPrerequisites,RPhosFate-method
-#'
-#' @inherit catchment return
-#'
-#' @seealso [`firstRun`], [`subsequentRun`]
-#'
-#' @examples
-#' \donttest{
-#' # temporary demonstration project copy
-#' cv_dir <- demoProject()
-#' # load temporary demonstration project
-#' x <- RPhosFate(
-#'   cv_dir = cv_dir,
-#'   ls_ini = TRUE
-#' )
-#'
-#' x <- transportCalcOrder(x)}
-#'
-#' @aliases transportCalcOrder
-#'
-#' @export
-setMethod(
-  "transportCalcOrder",
-  "RPhosFate",
-  function(x) {
-    compareRaster(
-      x@topo@rl_acc_wtd,
-      x@topo@rl_acc,
-      x@topo@rl_cha
-    )
-
-    # Overland flow accumulation
-    rl_acc_ovl <- x@topo@rl_acc
-    rl_acc_ovl[!is.na(x@topo@rl_cha)] <- NA_integer_
-
-    # Channel flow accumulation
-    rl_acc_cha <- x@topo@rl_acc
-    rl_acc_cha[ is.na(x@topo@rl_cha)] <- NA_integer_
-
-    # Transport calculation order as column-major index
-    im_acc_ovl <- as.matrix(rl_acc_ovl)
-    im_acc_cha <- as.matrix(rl_acc_cha)
-    ar_ord_ovl <- tapply(seq_along(im_acc_ovl), im_acc_ovl, identity)
-    ar_ord_cha <- tapply(seq_along(im_acc_cha), im_acc_cha, identity)
-
-    # Row order from index
-    is_rws <- x@helpers@is_rws
-    fun <- function(x) {
-      (x + is_rws) - ceiling(x / is_rws) * is_rws
-    }
-    iv_ord_ovl_row <- as.integer(unlist(lapply(ar_ord_ovl, fun)))
-    iv_ord_cha_row <- as.integer(unlist(lapply(ar_ord_cha, fun)))
-
-    # Column order from index
-    fun <- function(x) {
-      ceiling(x / is_rws)
-    }
-    iv_ord_ovl_col <- as.integer(unlist(lapply(ar_ord_ovl, fun)))
-    iv_ord_cha_col <- as.integer(unlist(lapply(ar_ord_cha, fun)))
-
-    # Overland as well as channel row and column numbers for top-down
-    # computation (C++ has zero-based numbering)
-    x@helpers@order@iv_ord_row <- c(iv_ord_ovl_row, iv_ord_cha_row) - 1L
-    x@helpers@order@iv_ord_col <- c(iv_ord_ovl_col, iv_ord_cha_col) - 1L
-
-    # Reverse overland row and column numbers for bottom-up computation
-    # (C++ has zero-based numbering)
-    x@helpers@order@iv_ord_ovl_row_rev <- rev(iv_ord_ovl_row) - 1L
-    x@helpers@order@iv_ord_ovl_col_rev <- rev(iv_ord_ovl_col) - 1L
+    x@topo@rl_rip <- writeLayer(x, "rip", x@topo@rl_rip, "INT4S")
+    x@topo@rl_inl <- writeLayer(x, "inl", x@topo@rl_inl, "INT4S")
 
     x
   }
@@ -478,6 +366,10 @@ setGeneric(
 #' \cite{Engman, E.T., 1986. Roughness coefficients for routing surface runoff.
 #' Journal of Irrigation and Drainage Engineering 112, 39–53.}
 #'
+#' \cite{Molnár, P., Ramírez, J.A., 1998. Energy dissipation theories and
+#' optimal channel characteristics of river networks. Water Resources Research
+#' 34, 1809–1818. https://doi.org/10.1029/98WR00983}
+#'
 #' @seealso [`firstRun`], [`subsequentRun`]
 #'
 #' @examples
@@ -494,7 +386,6 @@ setGeneric(
 #' x <- erosion(x)
 #' x <- emission(x, "PP")
 #' x <- transportPrerequisites(x)
-#' x <- transportCalcOrder(x)
 #'
 #' x <- transport(x, "PP")}
 #'
@@ -506,20 +397,20 @@ setMethod(
   "RPhosFate",
   function(x, substance = "PP") {
     assertChoice(substance, slotNames(x@substances))
-    compareRaster(
-      x@topo@rl_acc_wtd,
-      x@topo@rl_cha,
-      x@topo@rl_dir,
-      x@topo@rl_inl,
-      x@topo@rl_rip,
+    compareGeom(
+      x@topo@rl_acc_inf,
+      x@topo@rl_dir_inf,
+      x@topo@rl_slp_cap,
       x@transport@rl_man,
       if (substance == "SS") {
         x@erosion@rl_ero
       } else {
         slot(x@substances, substance)@rl_xxe
       },
-      x@transport@rl_rhy,
-      x@topo@rl_slp_cap
+      x@topo@rl_cha,
+      x@topo@rl_rds,
+      x@topo@rl_rip,
+      x@topo@rl_inl
     )
     qassert(x@parameters@ns_dep_ovl, "N1[0,)", .var.name = "ns_dep_ovl")
     qassert(x@parameters@ns_dep_cha, "N1[0,)", .var.name = "ns_dep_cha")
@@ -539,36 +430,37 @@ setMethod(
     cs_dir_old <- setwd(file.path(x@cv_dir[1L], "Result"))
     on.exit(setwd(cs_dir_old))
 
-    li_tpt <- transportCpp(
+    li_tpt <- dinfTransportCpp(
+      nm_acc_inf = as.matrix(x@topo@rl_acc_inf, wide = TRUE),
+      nm_dir_inf = as.matrix(x@topo@rl_dir_inf, wide = TRUE),
+      nm_slp_cap = as.matrix(x@topo@rl_slp_cap, wide = TRUE),
+      nm_man = as.matrix(x@transport@rl_man, wide = TRUE),
+      nm_xxe = if (substance == "SS") {
+        as.matrix(x@erosion@rl_ero, wide = TRUE)
+      } else {
+        as.matrix(slot(x@substances, substance)@rl_xxe, wide = TRUE)
+      },
+      im_cha = as.matrix(x@topo@rl_cha, wide = TRUE),
+      im_rds = as.matrix(x@topo@rl_rds, wide = TRUE),
+      im_rip = as.matrix(x@topo@rl_rip, wide = TRUE),
+      im_inl = as.matrix(x@topo@rl_inl, wide = TRUE),
+      substance = substance,
       parameters = x@parameters,
-      ns_dep_ovl = if (substance == "SS") {
-        x@parameters@ns_dep_ovl
-      } else {
-        x@parameters@ns_dep_ovl / x@parameters@nv_enr_rto[substance]
-      },
-      ns_tfc_inl = x@parameters@nv_tfc_inl[substance],
-      helpers    = x@helpers,
-      order      = x@helpers@order,
-      im_cha     = as.matrix(x@topo@rl_cha),
-      im_dir     = as.matrix(x@topo@rl_dir),
-      im_inl     = as.matrix(x@topo@rl_inl),
-      im_rip     = as.matrix(x@topo@rl_rip),
-      nm_man     = as.matrix(x@transport@rl_man),
-      nm_xxe     = if (substance == "SS") {
-        as.matrix(x@erosion@rl_ero)
-      } else {
-        as.matrix(slot(x@substances, substance)@rl_xxe)
-      },
-      nm_rhy     = as.matrix(x@transport@rl_rhy),
-      nm_slp     = as.matrix(x@topo@rl_slp_cap)
+      helpers = x@helpers,
+      is_ths = x@is_ths
     )
 
-    slot(x@substances, substance)@rl_xxr     <- writeLayer(x, "xxr"    , raster(li_tpt$nm_xxr    , template = x@topo@rl_acc_wtd), "FLT8S", substance)
-    slot(x@substances, substance)@rl_xxt_inp <- writeLayer(x, "xxt_inp", raster(li_tpt$nm_xxt_inp, template = x@topo@rl_acc_wtd), "FLT8S", substance)
-    slot(x@substances, substance)@rl_xxt_out <- writeLayer(x, "xxt_out", raster(li_tpt$nm_xxt_out, template = x@topo@rl_acc_wtd), "FLT8S", substance)
-    slot(x@substances, substance)@rl_xxt     <- writeLayer(x, "xxt"    , raster(li_tpt$nm_xxt    , template = x@topo@rl_acc_wtd), "FLT8S", substance)
-    slot(x@substances, substance)@rl_xxt_ctf <- writeLayer(x, "xxt_ctf", raster(li_tpt$nm_xxt_ctf, template = x@topo@rl_acc_wtd), "FLT8S", substance)
-    slot(x@substances, substance)@rl_xxt_cld <- writeLayer(x, "xxt_cld", raster(li_tpt$nm_xxt_cld, template = x@topo@rl_acc_wtd), "FLT8S", substance)
+    # begin debugging # nolint start
+    # writeRaster(rast(li_tpt$im_ifl    , crs = x@helpers@cs_cmt, extent = x@helpers@ex_cmt), filename = "ifl.tif"    , datatype = "INT4S", overwrite = TRUE)
+    # writeRaster(rast(li_tpt$im_ord    , crs = x@helpers@cs_cmt, extent = x@helpers@ex_cmt), filename = "ord.tif"    , datatype = "INT4S", overwrite = TRUE)
+    # writeRaster(rast(li_tpt$nm_xxt_rip, crs = x@helpers@cs_cmt, extent = x@helpers@ex_cmt), filename = "xxt_rip.tif", datatype = "FLT8S", overwrite = TRUE)
+    # end debugging # nolint end
+    slot(x@substances, substance)@rl_xxr     <- writeLayer(x, "xxr"    , rast(li_tpt$nm_xxr    , crs = x@helpers@cs_cmt, extent = x@helpers@ex_cmt), "FLT8S", substance)
+    slot(x@substances, substance)@rl_xxt     <- writeLayer(x, "xxt"    , rast(li_tpt$nm_xxt    , crs = x@helpers@cs_cmt, extent = x@helpers@ex_cmt), "FLT8S", substance)
+    slot(x@substances, substance)@rl_xxt_inp <- writeLayer(x, "xxt_inp", rast(li_tpt$nm_xxt_inp, crs = x@helpers@cs_cmt, extent = x@helpers@ex_cmt), "FLT8S", substance)
+    slot(x@substances, substance)@rl_xxt_out <- writeLayer(x, "xxt_out", rast(li_tpt$nm_xxt_out, crs = x@helpers@cs_cmt, extent = x@helpers@ex_cmt), "FLT8S", substance)
+    slot(x@substances, substance)@rl_xxt_ctf <- writeLayer(x, "xxt_ctf", rast(li_tpt$nm_xxt_ctf, crs = x@helpers@cs_cmt, extent = x@helpers@ex_cmt), "FLT8S", substance)
+    slot(x@substances, substance)@rl_xxt_cld <- writeLayer(x, "xxt_cld", rast(li_tpt$nm_xxt_cld, crs = x@helpers@cs_cmt, extent = x@helpers@ex_cmt), "FLT8S", substance)
 
     x
   }
